import socket

from liquid_earth_api.config import LOCAL


def make_url(endpoint: str) -> str:
    return f"{BASE_URL}/{endpoint}"


BASE_URL = f"https://apim-liquidearth.azure-api.net/python" if not LOCAL else\
    f"http://{socket.gethostname()}.local:7071/api"


def handle_response(response) -> any:
    """ Helper function to handle common response logic. """
    if response.ok:
        content_type_ = response.headers['Content-Type']
        parsed_response = response.json() if 'application/json' in content_type_ else response.text
        return parsed_response
    else:
        raise ValueError(f"HTTP Error {response.status_code}: {response.reason} {response.text}")



from typing import Union, Optional
import subsurface
from . import utils_api
from ..data.schemas import AddDataPostData, AddNewSpacePostData
from ..modules.rest_client import rest_interface
from ..modules.blob_client import blob_interface

import os


def set_token(token: str):
    # Check that the first 3 characters are le-
    if token[:3] != 'le-':
        raise ValueError('The token is invalid. Generate a new token from the LiquidEarth WebApp')
    
    if not os.path.exists('.env'):
        with open('.env', 'w') as f:
            f.write(f'LIQUID_EARTH_TOKEN={token}\n')
    else:
        with open('.env', 'a') as f:
            f.write(f'LIQUID_EARTH_TOKEN={token}\n')


def upload_mesh_to_existing_space(space_name: str, data: subsurface.UnstructuredData, file_name: str,
                                  token: str, grab_link: bool = True) -> Union[bool, dict]:
    # * grab space
    available_projects = get_available_projects(token)
    # ? which type is actually returned here?a
    found_project = utils_api.find_space_item(
        all_projects=available_projects,
        space_name=space_name
    )

    return _upload_mesh_common(data, file_name, found_project, grab_link, token)


def upload_mesh_to_new_space(space_name: str, data: subsurface.UnstructuredData,
                             file_name: str, token: str, grab_link: bool = True) -> Union[bool, dict]:
    post_data = AddNewSpacePostData(spaceName=space_name)
    new_project = post_create_space(
        add_new_space=post_data,
        token=token
    )

    return _upload_mesh_common(data, file_name, new_project, grab_link, token)


def get_deep_link(post_data: AddDataPostData, token: str):
    response = rest_interface.get_deep_link(post_data, token)
    return response


# TODO: ? Can we cache this easily?
def get_available_projects(token: str):
    return rest_interface.get_available_projects(token)


def post_create_space(add_new_space: AddNewSpacePostData, token: str) -> dict:
    return rest_interface.post_create_space(add_new_space, token)


def post_add_data_to_space(unstructured_data: subsurface.UnstructuredData, post_data: AddDataPostData, token: str):
    response: dict = rest_interface.post_add_data_to_space(post_data, token)
    uploading_files_response = blob_interface.push_unstructured_data(
        unstructured_data=unstructured_data,
        sas_dict=response
    )
    return uploading_files_response


def _upload_mesh_common(data, file_name, found_project, grab_link, token):
    # * upload data
    post_data = AddDataPostData(
        spaceId=found_project["SpaceId"],
        ownerId=found_project["OwnerId"],
        dataType="static_mesh",
        fileName=file_name
    )
    post_add_data_to_space(
        unstructured_data=data,
        post_data=post_data,
        token=token
    )
    response = True
    # * Grab link
    if grab_link:
        response = get_deep_link(post_data, token)
    return response



from dotenv import load_dotenv

load_dotenv()

from liquid_earth_api.api.le_api import (
    upload_mesh_to_existing_space,
    upload_mesh_to_new_space,
    get_available_projects,
    get_deep_link
)



import subsurface as ss
import gempy as gp


def gempy_to_subsurface(geomodel: gp.data.GeoModel) -> ss.UnstructuredData:
    meshes: ss.UnstructuredData = geomodel.solutions.raw_arrays.meshes_to_subsurface()
    return meshes



def find_space_item(all_projects: list, space_name: str):
    found_project = None
    for project in all_projects:
        if project["Name"] == space_name:
            found_project = project
    if found_project is None:
        raise ValueError("Space not found in LiquidEarth user. Make sure the space exists or create a new one")
    return found_project



import os

from dotenv import dotenv_values, load_dotenv

from liquid_earth_api.data.schemas import AddDataPostData, AddNewSpacePostData
from liquid_earth_api.api import le_api, utils_api

load_dotenv()

user_token = os.environ.get("LIQUID_EARTH_API_TOKEN")

space_name = "Test upload from python2"


def test_upload_mesh_to_existing_space():
    import gempy as gp
    from gempy.core.data.enumerators import ExampleModel

    gempy_model = gp.generate_example_model(ExampleModel.ANTICLINE, compute_model=True)
    foo = le_api.upload_mesh_to_existing_space(
        space_name=space_name,
        data=gempy_model.solutions.raw_arrays.meshes_to_subsurface(),
        file_name="test2",
        token=user_token
    )
    pass


def test_upload_mesh_to_new_space():
    import gempy as gp
    from gempy.core.data.enumerators import ExampleModel

    gempy_model = gp.generate_example_model(ExampleModel.ANTICLINE, compute_model=True)
    foo = le_api.upload_mesh_to_new_space(
        space_name="new_space_name",
        data=gempy_model.solutions.raw_arrays.meshes_to_subsurface(),
        file_name="test2",
        token=user_token
    )
    pass


def test_get_available_projects():
    available_projects = le_api.get_available_projects(
        token=user_token
    )
    return len(available_projects)


def test_get_deeplink():
    _project: AddDataPostData = _get_test_project(space_name)

    deep_link = le_api.get_deep_link(
        post_data=_project,
        token=user_token
    )
    print(deep_link)


def test_new_space():
    data = AddNewSpacePostData(spaceName=space_name)
    bar = le_api.post_create_space(
        add_new_space=data,
        token=user_token
    )

    print(bar)


def test_upload_data_to_space():
    # * Getting available projects
    import gempy as gp
    from gempy.core.data.enumerators import ExampleModel

    gempy_model = gp.generate_example_model(ExampleModel.ANTICLINE, compute_model=True)
    foo = le_api.post_add_data_to_space(
        unstructured_data=gempy_model.solutions.raw_arrays.meshes_to_subsurface(),
        post_data=_get_test_project(space_name),
        token=user_token
    )
    pass


def _get_test_project(space_name: str) -> AddDataPostData:
    all_projects = le_api.get_available_projects(
        token=user_token
    )
    found_project = utils_api.find_space_item(all_projects, space_name)
    post_data = AddDataPostData(
        spaceId=found_project["SpaceId"],
        ownerId=found_project["OwnerId"],
        dataType="static_mesh",
        fileName="test"
    )
    return post_data



from liquid_earth_api.modules.rest_client._utils import BASE_URL, handle_response
from liquid_earth_api.data.schemas import AddNewSpacePostData, AddDataPostData
import requests
from dataclasses import asdict


def get_deep_link(post_data: AddDataPostData, token: str) -> dict:
    response = requests.post(
        url=f"{BASE_URL}/GetDeepLinkFromSpace",
        json=asdict(post_data),
        headers={"Authorization": f"{token}"}
    )
    return handle_response(response)  # Use helper to process the response


def post_add_data_to_space(post_data: AddDataPostData, token: str) -> dict:
    response = requests.post(
        url=f"{BASE_URL}/AddDataToSpace",
        json=asdict(post_data),
        headers={"Authorization": f"{token}"}
    )
    return handle_response(response)


def get_available_projects(token: str) -> list:
    response = requests.get(
        url=f"{BASE_URL}/GetAvailableProjects",
        headers={"Authorization": f"{token}"}
    )
    return handle_response(response)


def post_create_space(add_new_space: AddNewSpacePostData, token: str) -> dict:
    response = requests.post(
        url=f"{BASE_URL}/AddNewSpace",
        json=asdict(add_new_space),
        headers={"Authorization": f"{token}"}
    )
    return handle_response(response)



import json

from azure.storage.blob import BlobClient

import subsurface


def push_unstructured_data(unstructured_data: subsurface.UnstructuredData, sas_dict: dict):
    meshes = unstructured_data
    try:
        data, header = meshes.to_binary()
        blob_client = BlobClient.from_blob_url(sas_dict["sasForTestLe"])
        _ = blob_client.upload_blob(data, overwrite=True)
        blob_client = BlobClient.from_blob_url(sas_dict["sasForTestJson"])
        _ = blob_client.upload_blob(json.dumps(header), overwrite=True)
        return True
    except Exception as e:
        print(f"Error uploading files: {e}")
        ValueError("Error uploading files")
        



from dataclasses import asdict

import requests

from liquid_earth_api.modules.rest_client._utils import BASE_URL
from liquid_earth_api.data.schemas import AddDataPostData


def get_deep_link(post_data: AddDataPostData, token: str):
    response = requests.post(
        url=f"{BASE_URL}/GetDeepLinkFromSpace",
        json=asdict(post_data),
        headers={
                "Authorization": f"{token}"
        }
    )

    # if request 200 return the deep link from text/plain
    if response.status_code == 200:
        return response.text
    else:
        print("Error getting deep link")
        return False



from dataclasses import dataclass


@dataclass
class AddDataPostData:
    spaceId: str
    ownerId: str
    dataType: str
    fileName: str
    
    
@dataclass
class AddNewSpacePostData:
    spaceName: str


import os
from os import path

from setuptools import setup, find_packages


def read_requirements(file_name, base_path=""):
    # Construct the full path to the requirements file
    full_path = os.path.join(base_path, file_name)
    requirements = []
    with open(full_path, "r", encoding="utf-8") as f:
        for line in f:
            # Strip whitespace and ignore comments
            line = line.strip()
            if line.startswith("#") or not line:
                continue

            # Handle -r directive
            if line.startswith("-r "):
                referenced_file = line.split()[1]  # Extract the file name
                # Recursively read the referenced file, making sure to include the base path
                requirements.extend(read_requirements(referenced_file, base_path=base_path))
            else:
                requirements.append(line)

    return requirements


setup(
    name='liquid_earth_api',
    packages=find_packages(exclude=('test', 'docs', 'examples')),
    url='',
    license='EUPL-v1.2',
    author='Miguel de la Varga',
    author_email='miguel@terranigma-solutions.com',
    description='Python API to interact with Liquid Earth',
    install_requires=read_requirements("requirements.txt", "requirements"),
    setup_requires=['setuptools_scm'],
    use_scm_version={
            "root"            : ".",
            "relative_to"     : __file__,
            "write_to"        : path.join("liquid_earth_api", "_version.py"),
            "fallback_version": "0.0.1"
    },
)
